#include <stdio.h>
#include <stdlib.h>

#define IN_ARG  1
#define OUT_ARG 2
#define KEY_ARG 3

#define MAX_FIELD_WIDTH 40

int main(int argc, char** argv)
{
  FILE* in, * out;
  char* labels;
  int numlabels, i;
  char c;
  char* str;
  
  if (argc != 4)
  {
    fprintf(stderr, "Usage:  %s [input file] [output file] [transformation key file]\n", argv[0]);
    exit(EXIT_SUCCESS);
  }
  
  in = fopen(argv[KEY_ARG], "r");
  if (in == NULL)
  {
    perror("Error opening key file");
    exit(EXIT_FAILURE);
  }
  
  str = (char*) malloc(MAX_FIELD_WIDTH);
  if (str == NULL)
  {
    perror("Error allocating temporary string");
    exit(EXIT_FAILURE);
  }

  //fscanf(in, "%*s %s", str);
  //printf("%d %s\n", 0, str);
  //fscanf(in, "%*s %s", str);
  //printf("%d %s\n", 1, str);
  //exit(0);
  numlabels = 0;
  c = fgetc(in);
  while (c != EOF)
  {
    if (c == '\n')
      numlabels++;
    c = fgetc(in);
  }
  rewind(in);
  
  labels = (char*) malloc(numlabels * MAX_FIELD_WIDTH);
  if (labels == NULL)
  {
    perror("Error allocating array for vertex labels");
    exit(EXIT_FAILURE);
  }
  
  for (i = 0; i < numlabels; i++)
    fscanf(in, "%*s %s", labels + i * MAX_FIELD_WIDTH);
  fclose(in);
  
  in = fopen(argv[IN_ARG], "r");
  if (in == NULL)
  {
    perror("Error opening input file");
    exit(EXIT_FAILURE);
  }
  
  out = fopen(argv[OUT_ARG], "w");
  if (out == NULL)
  {
    perror("Error opening output file");
    exit(EXIT_FAILURE);
  }
  
  i = 0;
  c = fgetc(in);
  while (c != EOF)
  {
    fflush(stdout);
    if (c >= '0' && c <= '9')
      str[i++] = c;
    else if (i > 0)
    {
      str[i] = '\0';
      i = atoi(str);
      if (i < 0 || i >= numlabels)
      {
        fprintf(stderr, "Error:  trying to access label %d of %d\n", i + 1, numlabels);
        exit(EXIT_FAILURE);
      }
      fprintf(out, "%s", labels + i * MAX_FIELD_WIDTH);
      i = 0;
      fputc(c, out);
    }
    else
      fputc(c, out);
    c = fgetc(in);
  }
  
  fclose(in);
  fclose(out);
  
  free(str);
  free(labels);
  
  exit(EXIT_SUCCESS);
}
