#ifndef __GRAPH_H
#define __GRAPH_H

#define INACTIVE     0x0
#define ACTIVE       0x1
#define QUERY_VERTEX 0x2
#define IN_QC        0x4
#define TO_REMOVE    0x8
#define QUERIED      0x10

typedef int graph_enum_t;     /* vertices are enumerated 0..nvertices-1 */

typedef unsigned long graph_matrix_t;  /* adjacency matrix base data type */

typedef struct
{
  graph_enum_t vertex;
} Edge;

typedef struct
{
  graph_enum_t active;       /* flag if vertex is in use */
  graph_enum_t degree;       /* number of edges for this vertex */
  graph_enum_t n2_size;      /* size of N^2(v) - N(v) - {v} */
  Edge* edge_list;           /* connectivity of this vertex */
  graph_enum_t* n2_list;     /* listing of N^2(v) - N(v) - {v} */
  graph_enum_t* hash_list;   /* edge hash table for quick lookup */
  graph_enum_t hash_size;    /* size of the hash table */
  graph_enum_t hash_mask;    /* mask for simple hash function */
} Vertex;

typedef struct
{
  graph_enum_t num_vertices;         /* total number of vertices in graph */
  graph_enum_t num_edges;            /* total number of edges */
  graph_enum_t num_n2s;
  graph_enum_t num_active_vertices;  /* number of active vertices */
  graph_enum_t max_degree;           /* highest degree of any vertex */
  graph_enum_t max_n2_size;          /* highest n2_size */
  graph_enum_t graph_size;           /* graph size in bytes */
  int edge_offset;                   /* offset to start of edge memory */
  graph_matrix_t** adjacency_matrix; /* connectivity bit matrix */
  graph_matrix_t** n2_matrix;
  graph_enum_t* hash_table;          /* connectivity hash table */
  graph_enum_t* n2_list;             /* N^2 listing */
  Edge* edge_array;                  /* start of graph memory for edges */
  
  Vertex vertex[];                   /* list of vertices (must be last in struct) */
  /* Edges follow vertices, and should be suitably aligned. */
} Graph;

Graph* make_graph(graph_enum_t nvertices, graph_enum_t nedges);
int connected(Graph* G, graph_enum_t u, graph_enum_t v);
void delete_lower_degree_vertices(Graph* G, graph_enum_t k);
void free_graph(Graph* G);
Graph* read_graph(const char* filename, long matrix_cutoff, long hash_cutoff);
void print_graph(Graph *G);

#endif  /* __GRAPH_H */
