#include <string.h>
#include "graph.old.h"

/* Malloc and initialize a graph, returns a pointer to it */
Graph *make_graph(unsigned int num_vertices)
{
    Graph *G;
    int i;

    if ( (G = (Graph *) malloc(sizeof(Graph))) == NULL ) {
	perror("malloc");
	exit(-1);
    }

    G->_num_vertices = num_vertices;
    G->_num_active_vertices = num_vertices;
    G->_num_edges = 0;

    if ( (G->_vertex = (Vertex **)malloc(num_vertices*sizeof(Vertex *))) == NULL ) {
	perror("malloc error\n");
	exit(-1);
    }

    for (i=0; i<num_vertices; i++) {
	if ( (G->_vertex[i] = (Vertex *)malloc(sizeof(Vertex))) == NULL ) {
	    perror("malloc error\n");
	    exit(-1);
	}
	G->_vertex[i]->_degree = 0;
	G->_vertex[i]->_active = 1;
	G->_vertex[i]->_edge_list = NULL;
	G->_vertex[i]->_tail = NULL;
    }

    return G;
}



/**
 *
 *
 *
 */
void delete_lower_degree_vertices(Graph *G, unsigned int k)
{
    int i;

    for (i=0; i<G->_num_vertices; i++) {
	if ( G->_vertex[i]->_degree < k-1 ) {
	    G->_vertex[i]->_active = 0;
	    G->_num_active_vertices--;
	}
    }
}


/* Free the memory of a graph */
void free_graph(Graph *G)
{
    int i;

    for (i=0; i<G->_num_vertices; i++) {
	free_vertex(G->_vertex[i]);
    }

    free(G);
}


void free_vertex(Vertex *vertex)
{
    Edge *edge = vertex->_edge_list;
    Edge *tmp;

    while ( edge != NULL ) {
	tmp = edge;
	edge = edge->_next;
	free(tmp);
    }
}


void add_edge(Graph *G, unsigned int u, unsigned int v)
{
    if ( G->_vertex[u]->_tail == NULL ) {
	if ( (G->_vertex[u]->_edge_list = (Edge *)malloc(sizeof(Edge *))) == NULL ) {
	    perror("malloc error\n");
	    exit(-1);
	}
	G->_vertex[u]->_tail = G->_vertex[u]->_edge_list;
    }
    else {
	if ( (G->_vertex[u]->_tail->_next = (Edge *)malloc(sizeof(Edge *))) == NULL ) {
	    perror("malloc error\n");
	    exit(-1);
	}
	G->_vertex[u]->_tail = G->_vertex[u]->_tail->_next;
    }

    G->_vertex[u]->_tail->_vertex = v;
    G->_vertex[u]->_tail->_next = NULL;
    G->_vertex[u]->_degree++;
}


/* Read in an unweighted edge-list file, return a pointer to the graph */
Graph * read_graph(FILE *fp)
{
    unsigned int u, v, n, e, i;
    Graph *G;
    fscanf(fp, "%d %d", &n, &e);
    G = make_graph(n);
    G->_num_edges = e;

    for (i = 0; i < e; i++) {
	fscanf(fp, "%d", &u);
	fscanf(fp, "%d", &v);
	add_edge(G, u, v);
	add_edge(G, v, u);
    }
    return G;
}


Graph * read_addition_graph(FILE *fp)
{
    unsigned int u, v, n, e, i;
    Graph *G;
    fscanf(fp, "%d %d %*d", &n, &e);
    G = make_graph(n);
    G->_num_edges = e;

    for (i = 0; i < e << 1; i++) {
	fscanf(fp, "%d", &u);
	fscanf(fp, "%d", &v);
	add_edge(G, u, v);
    }
    return G;
}


void print_graph(Graph *G)
{
    unsigned int i;
    Edge *edge;

    for (i=0; i<G->_num_vertices; i++) {
	edge = G->_vertex[i]->_edge_list;
	printf("%d: ", i);
	while ( edge != NULL ) {
	    printf("%d ",edge->_vertex);
	    edge = edge->_next;
	}
	printf("\n");
    }
}
