#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//#define DEBUG

int main(int argc, char** argv)
{
  if (argc > 3)
  {
    printf("%s [GTgraph output] [output graph file]\n", argv[0]);
    printf("%s [GTgraph output] :  write to console\n", argv[0]);
    printf("%s : read from and write to console\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  FILE* in = stdin;
  FILE* out = stdout;
  if (argc > 1)
  {
    in = fopen(argv[1], "r");
    if (in == NULL)
    {
      fprintf(stderr, "Error opening input file (%s", argv[1]);
      perror(")");
      exit(EXIT_FAILURE);
    }
  }
  if (argc > 2)
  {
    out = fopen(argv[2], "w");
    if (out == NULL)
    {
      fprintf(stderr, "Error opening output file (%s", argv[2]);
      perror(")");
      exit(EXIT_FAILURE);
    }
  }
  
  char* string = (char*) malloc(81);
  if (string == NULL)
  {
    perror("Error allocating temporary string");
    exit(EXIT_FAILURE);
  }
  
  int temp;
  double flpt;
  
beginning:
  fscanf(in, "%s", string);
  while (string != NULL && strcmp(string, "p"))
    fscanf(in, "%s", string);
  if (string == NULL)
  {
    printf("Could not find \"p sp [# vertices] [# edges]\" in input file.\n");
    exit(EXIT_FAILURE);
  }
  
  fscanf(in, "%s", string);
  while (string != NULL && strcmp(string, "p") == 0)
    fscanf(in, "%s", string);
  if (strcmp(string, "sp") != 0)
    goto beginning;
  
  int nverts = 0, nedges = 0;
  fscanf(in, "%d %d", &nverts, &nedges);
  
  #ifdef DEBUG
  printf("%d vertices, %d (duped) edges\n", nverts, nedges);
  #endif
  
  int* graph = (int*) malloc((nverts * (nverts - 1) >> 1) * sizeof(int));
  int* verts = (int*) calloc(nverts, sizeof(int));
  if (graph == NULL || verts == NULL)
  {
    perror("Error allocating graph structure");
    exit(EXIT_FAILURE);
  }
  
  int i, j, u, v, w, t, index;
  for (i = nedges; i > 0; i--)
  {
    fscanf(in, "%s", string);
    if (strcmp(string, "a"))
    {
      fprintf(stderr, "Error:  expected 'a', found '%s'\n", string);
      exit(EXIT_FAILURE);
    }
    
    fscanf(in, "%d %d %d", &u, &v, &w);
    if (u > v)
    {
      t = u;
      u = v;
      v = t;
    }
    else if (u == v)
    {
      fprintf(stderr, "Self-loop as edge %d:  (%d, %d)\n", i, u, v);
      //exit(EXIT_FAILURE);
      nedges--;
      continue;
    }
    index = (u * (nverts + nverts - u - 1) >> 1) + v - u - 1;
    if (graph[index] == 0)
    {
      #ifdef DEBUG
      printf("(%d, %d) <=> [%d] => %d\n", u, v, index, w + 1);
      printf("verts[%d] = verts[%d] = -1\n", u, v);
      #endif
      graph[index] = w + 1;
      verts[u] = -1;
      verts[v] = -1;
    }
    else
    {
      #ifdef DEBUG
      printf("dupe:  (%d, %d) <=> [%d] == %d\n", u, v, index, graph[index]);
      #endif
      nedges--;
    }
  }
  
  #ifdef DEBUG
  for (i = 0; i < nverts; i++)
    if (verts[i] != 0 && verts[i] != -1)
      printf("verts[%d] = %d!\n", i, verts[i]);
  #endif
  t = 0;
  for (i = 0; i < nverts; i++)
    if (verts[i])
      verts[i] = t++;
    else
      verts[i] = -999;
  
  #ifdef DEBUG
  for (i = 0; i < nverts; i++)
  {
    if (verts[i])
      printf("Vertex %d is #%d\n", i, verts[i]);
    else
      printf("Vertex %d does not exist\n", i);
  }
  printf("Total vertices = %d\n", t);
  #endif
  
  fprintf(out, "%d %d\n", t, nedges);
  for (i = index = 0; i < nverts; i++)
    for (j = i + 1; j < nverts; j++)
    {
      if (graph[index])
      {
        #ifdef DEBUG
        printf("%d %d %.2f -> %d %d %.2f\n", i, j, graph[index] / 100.0, verts[i], verts[j], graph[index] / 100.0);
        #endif
        fprintf(out, "%d %d\n", verts[i], verts[j]);
      }
      index++;
    }
  
  exit(EXIT_SUCCESS);
}
