#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//#define DEBUG

typedef unsigned char index_t;

#define BRANCH_FACTOR (1 << (sizeof(index_t) + 2))
#define BASE_LEVELS   2

int* clique;
int clique_size = 0;
int max_vertex = 0;
int* delta;
int num_qcs = 0;
int maxlevel = BASE_LEVELS;
index_t* qc_index;

int max_index(int level, int word)
{
  int block = (word + 1) * delta[level] - 1;
  while (word >= BRANCH_FACTOR)
  {
    word /= BRANCH_FACTOR;
    block += word;
  }
  
  #ifdef DEBUG
  if (block >= delta[maxlevel] | block < 0)
  {
    printf("Tried to access block %d (%d, %d) out of %d\n", block, level, word, delta[maxlevel]);
    exit(EXIT_FAILURE);
  }
  #endif
  
  return block;
}

void read_qc()
{
  int i;
  #ifdef DEBUG
  printf("CLIQUE %d (%d | %d)\n", num_qcs, num_qcs / BRANCH_FACTOR, 1 << (num_qcs % BRANCH_FACTOR));
  #endif
  
  if (num_qcs == delta[maxlevel] - delta[maxlevel - 1])
  {
    #ifdef DEBUG
    printf("Reallocating maximality index!\n");
    printf("Old delta: ");
    for (i = 0; i <= maxlevel; i++)
      printf(" %d", delta[i]);
    #endif
    //Reallocate
    char* temp = (char*) malloc((maxlevel + 2) * sizeof(int));
    if (temp == NULL)
    {
      perror("Error enlarging maximality index helper array");
      exit(EXIT_FAILURE);
    }
    memcpy(temp, delta, (maxlevel + 1) * sizeof(int));
    free(delta);
    delta = (int*) temp;
    delta[maxlevel + 1] = delta[maxlevel] * BRANCH_FACTOR + 1;
    #ifdef DEBUG
    printf("\nNew delta: ");
    for (i = 0; i <= maxlevel; i++)
      printf(" %d", delta[i]);
    printf(" %d\n", delta[maxlevel + 1]);
    #endif
    
    temp = (char*) malloc(delta[maxlevel + 1] * max_vertex * sizeof(index_t));
    if (temp == NULL)
    {
      perror("Error enlarging quasi-clique maximality index");
      exit(EXIT_FAILURE);
    }
    memcpy(temp, qc_index, delta[maxlevel] * max_vertex * sizeof(index_t));
    memset(temp + delta[maxlevel] * max_vertex, 0, (delta[maxlevel + 1] - delta[maxlevel]) * max_vertex * sizeof(index_t));
    free(qc_index);
    qc_index = (index_t*) temp;
    
    maxlevel++;
  }
  
  int word, pos, level, idx;
  word = num_qcs / BRANCH_FACTOR;
  pos  = num_qcs % BRANCH_FACTOR;
  for (level = 0; level <= maxlevel; level++)
  {
    idx = max_index(level, word);
    for (i = 0; i < clique_size; i++)
    {
      #ifdef DEBUG
      printf("qc_index[%d][%d, %d]:  %d -> ", clique[i], level, word, qc_index[idx * max_vertex + clique[i]]);
      #endif
      qc_index[idx * max_vertex + clique[i]] |= (1 << pos);
      #ifdef DEBUG
      printf("%d\n", qc_index[idx * max_vertex + clique[i]]);
      #endif
    }
    
    pos  = word % BRANCH_FACTOR;
    word = word / BRANCH_FACTOR;
  }
  num_qcs++;
}

index_t check_max(int level, int word)
{
  int idx = max_index(level, word);
  #ifdef DEBUG
  printf("%*sChecking byte[%d, %d]...\n", maxlevel - level + 1, "", level, word);
  #endif
  int i;
  index_t temp;
  temp = qc_index[idx * max_vertex + clique[clique_size - 1]];
  #ifdef DEBUG
  printf("%*sqc_index[%d][%d, %d] = %d\n", maxlevel - level + 1, "", clique[clique_size - 1], level, word, qc_index[idx * max_vertex + clique[clique_size - 1]]);
  #endif
  for (i = 0; (i < clique_size) & (temp != 0); i++)
  {
    #ifdef DEBUG
    printf("%*sqc_index[%d][%d, %d] = %d\n", maxlevel - level + 1, "", clique[i], level, word, qc_index[idx * max_vertex + clique[i]]);
    #endif
    temp &= qc_index[idx * max_vertex + clique[i]];
  }
  
  if (level == 0)
  {
    #ifdef DEBUG
    printf("%*sByte[%d, %d] %s\n", maxlevel - level + 1, "", level, word, temp? "not cleared":"cleared");
    #endif
    if (temp == 0)
      return 0;
    
    i = 0;
    while ((temp & 1) == 0)
    {
      temp >>= 1;
      i++;
    }
    
    #ifdef DEBUG
    //printf("Returning %d = %d * %d + %d\n", word * BRANCH_FACTOR + i, word, BRANCH_FACTOR, i);
    #endif
    
    return word * BRANCH_FACTOR + i;
  }
  
  #ifdef DEBUG
  printf("Checking temp = %d\n", temp);
  #endif
  if (temp != 0)
  {
    index_t mask;
    for (mask = 1, i = word * BRANCH_FACTOR; mask != 0; mask <<= 1, i++)
    {
      if ((temp & mask) != 0)
      {
        int ret = check_max(level - 1, i);
        if (ret)
        {
          #ifdef DEBUG
          printf("%*sFailed on bit %d (%d)!\n", maxlevel - level + 1, "", i - word * BRANCH_FACTOR, ret);
          #endif
          return ret;
        }
      }
    }
  }
  
  #ifdef DEBUG
  printf("%*sByte[%d, %d] clear!\n", maxlevel - level + 1, "", level, word);
  #endif
  return 0;
}

int check_maximality()
{
  #if (TIME_CONST & TIME_MAX)
  pause_clock();
  select_clock(&maxtime);
  #endif
  #ifdef DEBUG
  printf("Checking maximality: ");
  //f;
  #endif
  //update_max(maxlevel, 0);
  index_t retval = check_max(maxlevel, 0);
  
  #if (TIME_CONST & TIME_MAX)
  resume_clock();
  #endif
  return retval;
}

int main(int argc, char** argv)
{
  if (argc != 2)
  {
    printf("Usage:  %s [clique file]\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  delta = (int*) malloc((BASE_LEVELS + 1) * sizeof(int));
  if (delta == NULL)
  {
    perror("Error allocating helper array for maximality index");
    exit(EXIT_FAILURE);
  }
  
  FILE* in = fopen(argv[1], "r");
  
  int vertex = -1;
  int max_verts = 0;
  int num_cliques = 0;
  char ch = getc(in);
  int i, dupe;
  
  #ifdef DEBUG
  printf("Initializing...\n");
  #endif
  while (ch != EOF)
  {
    if (ch == '\n')
    {
      if (vertex > max_vertex)
        max_vertex = vertex;
      if (++clique_size > max_verts)
        max_verts = clique_size;
      vertex = -1;
      clique_size = 0;
      num_cliques++;
    }
    else if (ch == ' ')
    {
      clique_size++;
      if (vertex > max_vertex)
        max_vertex = vertex;
      vertex = -1;
    }
    else if (vertex != -1)
      vertex = vertex * 10 + ch - '0';
    else
      vertex = ch - '0';
    ch = getc(in);
  }
  rewind(in);
  max_vertex++;
  
  #ifdef DEBUG
  printf("max_clique_size: %d\nnum_cliques: %d\ngraph_vertices: %d\n", max_verts, num_cliques, max_vertex);
  printf("Delta:  1");
  #endif
  delta[0] = 1;
  for (i = 1; i <= BASE_LEVELS; i++)
  {
    delta[i] = BRANCH_FACTOR * delta[i - 1] + 1;
    #ifdef DEBUG
    printf(" %d", delta[i]);
    #endif
  }
  #ifdef DEBUG
  printf("\n");
  #endif
  
  qc_index = (index_t*) calloc(delta[BASE_LEVELS] * max_vertex, sizeof(index_t));
  if (qc_index == NULL)
  {
    perror("Error allocating quasi-clique maximality index");
    exit(EXIT_FAILURE);
  }
  
  clique = malloc(max_verts * sizeof(int));
  if (clique == NULL)
  {
    perror("Error allocating clique array");
    exit(EXIT_FAILURE);
  }
  
  //Start algorithm
  #ifdef DEBUG
  printf("Reading in cliques\n");
  #endif
  for (i = 0; i < num_cliques; i++)
  {
    clique_size = 0;
    vertex = -1;
    ch = getc(in);
    while (ch != '\n')
    {
      if (ch == ' ')
      {
        clique[clique_size++] = vertex;
        vertex = -1;
      }
      else if (vertex == -1)
        vertex = ch - '0';
      else
        vertex = vertex * 10 + ch - '0';
      ch = getc(in);
    }
    clique[clique_size++] = vertex;
    #ifdef DEBUG
    printf("CLQ %d (%d): ", i, clique_size);
    int j;
    for (j = 0; j < clique_size; j++)
      printf(" %d", clique[j]);
    printf("\n");
    #endif

    read_qc();
  }
  rewind(in);
  
  #ifdef DEBUG
  printf("Checking maximality...\n");
  #endif
  for (i = 0; i < num_cliques; i++)
  {
    clique_size = 0;
    vertex = -1;
    ch = getc(in);
    while (ch != '\n')
    {
      if (ch == ' ')
      {
        clique[clique_size++] = vertex;
        vertex = -1;
      }
      else if (vertex == -1)
        vertex = ch - '0';
      else
        vertex = vertex * 10 + ch - '0';
      ch = getc(in);
    }
    clique[clique_size++] = vertex;

    qc_index[max_index(0, i / BRANCH_FACTOR) * max_vertex + clique[0]] &= ~(1 << i % BRANCH_FACTOR);

    #ifdef DEBUG
    printf("qc_index[%d][0, %d] &= ~%d = 1 << %d = %d %% %d\n", clique[0], max_index(0, i / BRANCH_FACTOR), 1 << i % BRANCH_FACTOR, i % BRANCH_FACTOR, i, BRANCH_FACTOR);
    printf("CLQ %d (%d): ", i + 1, clique_size);
    int j;
    for (j = 0; j < clique_size; j++)
      printf(" %d", clique[j]);
    printf("\n");
    #endif
    dupe = check_maximality();
    
    qc_index[max_index(0, i / BRANCH_FACTOR) * max_vertex + clique[0]] |= (1 << i % BRANCH_FACTOR);
    if (dupe)
      printf("Mu, gamma-quasi-clique #%d is a subset of #%d\n", i + 1, dupe + 1);
    //else
      //printf("Mu, gamma-quasi-clique #%d is maximal!\n", i);
  }
  
  exit(EXIT_SUCCESS);
}
