#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "graph.old.h"
#include "lookup3.h"

#define MAX_FIELD_SIZE 20

typedef struct sepchain
{
  int id;
  char* str;
  struct sepchain* next;
} LL;

void output_graph(Graph *G, FILE* write)
{
  unsigned int i;
  Edge *edge;

  for (i=0; i<G->_num_vertices; i++)
  {
    edge = G->_vertex[i]->_edge_list;
    //fprintf(write, "%d: ", i);
    while ( edge != NULL )
    {
      fprintf(write, "%d %d\n", i, edge->_vertex);
      edge = edge->_next;
    }
    //fprintf(write, "\n");
  }
}

int main(int argc, char** argv)
{
  FILE* in, * out, * key;
  char** labels;
  char* e1, * e2, * e3;
  char c;
  int v1, v2, i, lines, hash, hashbits, t, mid, maxid;
  int maxv;
  LL** ht;
  LL* check;
  LL* last;
  Graph* G;

  if (argc != 3 && argc != 4)
  {
    fprintf(stderr, "Usage:  %s <input raw graph file> <output weighted graph file> [output key file]\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  in = fopen(argv[1], "r");
  if (in == NULL)
  {
    perror("Error opening input file");
    exit(EXIT_FAILURE);
  }
  out = fopen(argv[2], "w+");
  if (out == NULL)
  {
    perror("Error opening output file");
    exit(EXIT_FAILURE);
  }
  if (argc == 4)
  {
    key = fopen(argv[3], "w+");
    if (key == NULL)
    {
      perror("Error opening key output file");
      exit(EXIT_FAILURE);
    }
  }
  else
    key = NULL;
  
  e1 = (char*) malloc(MAX_FIELD_SIZE);
  e2 = (char*) malloc(MAX_FIELD_SIZE);
  if (e1 == NULL || e2 == NULL)
  {
    perror("Error allocating temporary storage strings");
    exit(EXIT_FAILURE);
  }
  
  lines = 0;
  c = fgetc(in);
  while (c != EOF)
  {
    if (c == '\n')
      lines++;
    c = fgetc(in);
  }
  rewind(in);
  
  #ifdef DEBUG
  printf("%d lines\n", lines);
  #endif
  
  if (key != NULL)
  {
    labels = (char**) malloc(lines * sizeof(char*));
    if (labels == NULL)
    {
      perror("Error allocating key array");
      exit(EXIT_FAILURE);
    }
  }

  t = lines;
  hashbits = 0;
  mid = 16;
  while (mid > 1)
  {
    if (t >> mid)
    {
      t = t >> mid;
      hashbits += mid;
    }
    mid >>= 1;
  }
  hashbits += t + 1;
  
  #ifdef DEBUG
  printf("(%d bits)\n", hashbits);
  #endif
  
  ht = (LL**) calloc(hashsize(hashbits), sizeof(LL*));
  if (ht == NULL)
  {
    perror("Error allocating hash table");
    exit(EXIT_FAILURE);
  }
  
  maxid = 0;
  for (i = 0; i < lines; i++)
  {
    fscanf(in, "%s %s\n", e1, e2);
    hash = hashlittle(e1, strlen(e1), 0) & hashmask(hashbits);
    
    #ifdef DEBUG
    printf("%s -> %d", e1, hash);
    #endif
    check = ht[hash];
    last = NULL;
    while (check != NULL && strcmp(check->str, e1) != 0)
    {
      last = check;
      check = check->next;
    }
    if (check == NULL)
    {
      #ifdef DEBUG
      printf(" (new id, %s, %d)", last == NULL?"new hash" :"repeat hash", maxid);
      #endif
      check = (LL*) malloc(sizeof(LL));
      check->str = (char*) malloc(MAX_FIELD_SIZE);
      if (check->str == NULL)
      {
        perror("Error allocating string in hash table");
        exit(EXIT_FAILURE);
      }
      strcpy(check->str, e1);
      check->id = maxid;
      if (key != NULL)
        labels[maxid] = check->str;
      maxid++;
      check->next = NULL;
      if (last != NULL)
        last->next = check;
      else
        ht[hash] = check;
    }
    v1 = check->id;
    #ifdef DEBUG
    printf(" -> %d\n", v1);
    #endif
    
    hash = hashlittle(e2, strlen(e2), 0) & hashmask(hashbits);
    #ifdef DEBUG
    printf("%s -> %d", e1, hash);
    #endif
    check = ht[hash];
    last = NULL;
    while (check != NULL && strcmp(check->str, e2) != 0)
    {
      last = check;
      check = check->next;
    }
    if (check == NULL)
    {
      #ifdef DEBUG
      printf(" (new id, %s, %d)", last == NULL?"new hash" :"repeat hash", maxid);
      #endif
      check = (LL*) malloc(sizeof(LL));
      check->str = (char*) malloc(MAX_FIELD_SIZE);
      if (check->str == NULL)
      {
        perror("Error allocating string in hash table");
        exit(EXIT_FAILURE);
      }
      strcpy(check->str, e2);
      check->id = maxid;
      if (key != NULL)
        labels[maxid] = check->str;
      maxid++;
      check->next = NULL;
      if (last != NULL)
        last->next = check;
      else
        ht[hash] = check;
    }
    v2 = check->id;
    #ifdef DEBUG
    printf(" -> %d\n", v2);
    #endif
  }
  fprintf(out, "%d %d\n", maxid, lines);
  rewind(in);
  
  G = make_graph(maxid);
  for (i = 0; i < lines; i++)
  {
    fscanf(in, "%s %s\n", e1, e2);
    hash = hashlittle(e1, strlen(e1), 0) & hashmask(hashbits);
    
    check = ht[hash];
    last = NULL;
    while (check != NULL && strcmp(check->str, e1) != 0)
    {
      last = check;
      check = check->next;
    }
    v1 = check->id;
    
    hash = hashlittle(e2, strlen(e2), 0) & hashmask(hashbits);
    check = ht[hash];
    last = NULL;
    while (check != NULL && strcmp(check->str, e2) != 0)
    {
      last = check;
      check = check->next;
    }
    v2 = check->id;
    
    //fprintf(out, "%d %d\n", v1, v2);
    add_edge(G, v1, v2);
  }
  
  if (key != NULL)
    for (i = 0; i < maxid; i++)
      fprintf(key, "%d %s\n", i, labels[i]);

  //printf("%d vertices, %d edges\n", maxid, lines);
  output_graph(G, out);
  fclose(in);
  fclose(out);
  if (key != NULL)
    fclose(key);
}
