#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lookup3.h"
#include "vertexset1c.h"

#define C_SRC      1
#define CNEW_SRC   2
#define CMINUS_SRC 4
#define CPLUS_SRC  8

//#define DEBUG

typedef struct linklist
{
  int source;
  PVertexSet clique;
  struct linklist* next;
} LL;

int main(int argc, char** argv)
{
  FILE* c, *cnew, *cminus, *cplus;
  char* cminusfile, * cplusfile;
  long maxcliques, num_ccliques, tc, i, cliquecount;
  int hashbits, len, vertex, mid, hash;
  LL** ht, ** ccliques;
  LL* check, * last;
  PVertexSet clique;
  PVertexSetIterator* iter;
  char ch;
  
  if (argc != 4)
  {
    fprintf(stderr, "Usage:  %s [C file] [C_NEW file] [C_PLUS/C_MINUS root name]\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  c = fopen(argv[1], "r");
  if (c == NULL)
  {
    perror("Error opening C file");
    exit(EXIT_FAILURE);
  }
  cnew = fopen(argv[2], "r");
  if (cnew == NULL)
  {
    perror("Error opening C_NEW file");
    exit(EXIT_FAILURE);
  }
  
  len = strlen(argv[3]);
  cminusfile = (char*) malloc(len + 8);
  cplusfile = (char*) malloc(len + 7);
  if (cminusfile == NULL || cplusfile == NULL)
  {
    perror("Error allocating file name strings");
    exit(EXIT_FAILURE);
  }
  strncpy(cminusfile, argv[3], len);
  strncpy(cminusfile + len, ".cminus", 8);
  strncpy(cplusfile, argv[3], len);
  strncpy(cplusfile + len, ".cplus", 7);
  
  cminus = fopen(cminusfile, "r");
  cplus = fopen(cplusfile, "r");
  if (cminus == NULL || cplus == NULL)
  {
    perror("Error opening C_PLUS/C_MINUS");
    exit(EXIT_FAILURE);
  }
  
  init();
  
  maxcliques = 0;
  ch = getc(c);
  while (ch != EOF)
  {
    if (ch == '\n')
      maxcliques++;
    ch = getc(c);
  }
  rewind(c);
  num_ccliques = maxcliques;
  
  ch = getc(cnew);
  while (ch != EOF)
  {
    if (ch == '\n')
      maxcliques++;
    ch = getc(cnew);
  }
  rewind(cnew);
  
  tc = maxcliques;
  hashbits = 0;
  mid = sizeof(long) << 2;
  while (mid > 1)
  {
    if (tc >> mid)
    {
      tc = tc >> mid;
      hashbits += mid;
    }
    mid >>= 1;
  }
  hashbits += tc + 1;
  
  #ifdef DEBUG
  printf("maxcliques:  %d, num_ccliques:  %d, hashbits:  %d\n", maxcliques, num_ccliques, hashbits);
  #endif
  
  ht = (LL**) malloc(hashsize(hashbits) * sizeof(LL*));
  if (ht == NULL)
  {
    perror("Error allocating hash table");
    exit(EXIT_FAILURE);
  }
  
  ccliques = (LL**) malloc(num_ccliques * sizeof(LL*));
  if (ccliques == NULL)
  {
    perror("Error allocating clique array");
    exit(EXIT_FAILURE);
  }
  
  #ifdef DEBUG
  printf("Reading in C_NEW\n");
  #endif
  ch = getc(cnew);
  vertex = -1;
  clique = newVertexSet();
  while (ch != EOF)
  {
    if (ch == '\n')
    {
      if (vertex >= 0)
      {
        #ifdef DEBUG
        //printf("%d", vertex);
        #endif
        clique = addVertex(getVertex(vertex, NULL), clique);
      }
      
      #ifdef DEBUG
      //printf("\n");
      #endif
      iter = getVertexIterator(clique);
      hash = 0;
      while (hasNext(iter))
      {
        vertex = getVertexId(getNext(iter), NULL);
        hash = hashlittle(&vertex, sizeof(int), hash);
      }
      hash &= hashmask(hashbits);
      free(iter);
      
      last = NULL;
      check = ht[hash];
      while (check != NULL && setEqual(check->clique, clique) == FALSE)
      {
        last = check;
        check = check->next;
      }
      if (check == NULL)
      {
        check = (LL*) malloc(sizeof(LL));
        if (check == NULL)
        {
          perror("Error allocating clique structure");
          exit(EXIT_FAILURE);
        }
        check->clique = clique;
        check->next = NULL;
        check->source = CNEW_SRC;
        if (last != NULL)
          last->next = check;
        else
          ht[hash] = check;
      }
      else
      {
        if (check->source & CNEW_SRC)
        {
          printf("Duplicate clique in C_NEW: ");
          printvs(clique);
        }
        check->source |= CNEW_SRC;
      }
      
      clique = newVertexSet();
      vertex = -1;
      cliquecount++;
    }
    else if (ch == ' ')
    {
      if (vertex >= 0)
      {
        #ifdef DEBUG
        //printf("%d", vertex);
        #endif
        clique = addVertex(getVertex(vertex, NULL), clique);
      }
      #ifdef DEBUG
      //printf(" ");
      #endif
      vertex = -1;
    }
    else if (ch >= '0' && ch <= '9')
    {
      if (vertex >= 0)
        vertex = vertex * 10 + ch - '0';
      else
        vertex = ch - '0';
    }
    ch = getc(cnew);
  }
  
  #ifdef DEBUG
  printf("Reading in C\n");
  #endif
  
  ch = getc(c);
  cliquecount = 0;
  vertex = -1;
  clique = newVertexSet();
  while (ch != EOF)
  {
    if (ch == '\n')
    {
      if (vertex >= 0)
        clique = addVertex(getVertex(vertex, NULL), clique);
      iter = getVertexIterator(clique);
      hash = 0;
      while (hasNext(iter))
      {
        vertex = getVertexId(getNext(iter), NULL);
        hash = hashlittle(&vertex, sizeof(int), hash);
      }
      hash &= hashmask(hashbits);
      free(iter);
      
      last = NULL;
      check = ht[hash];
      while (check != NULL && setEqual(check->clique, clique) == FALSE)
      {
        last = check;
        check = check->next;
      }
      if (check == NULL)
      {
        check = (LL*) malloc(sizeof(LL));
        if (check == NULL)
        {
          perror("Error allocating clique structure");
          exit(EXIT_FAILURE);
        }
        check->clique = clique;
        check->next = NULL;
        check->source = C_SRC;
        if (last != NULL)
          last->next = check;
        else
          ht[hash] = check;
      }
      else
      {
        if (check->source & C_SRC)
        {
          printf("Duplicate clique in C: ");
          printvs(clique);
        }
        check->source |= C_SRC;
      }
      
      clique = newVertexSet();
      vertex = -1;
      ccliques[cliquecount++] = check;
    }
    else if (ch == ' ')
    {
      if (vertex >= 0)
        clique = addVertex(getVertex(vertex, NULL), clique);
      vertex = -1;
    }
    else if (ch >= '0' && ch <= '9')
    {
      if (vertex >= 0)
        vertex = vertex * 10 + ch - '0';
      else
        vertex = ch - '0';
    }
    ch = getc(c);
  }
  
  #ifdef DEBUG
  printf("Reading in C_PLUS\n");
  #endif
  
  ch = getc(cplus);
  vertex = -1;
  clique = newVertexSet();
  while (ch != EOF)
  {
    if (ch == '\n')
    {
      if (vertex >= 0)
        clique = addVertex(getVertex(vertex, NULL), clique);
      iter = getVertexIterator(clique);
      hash = 0;
      while (hasNext(iter))
      {
        vertex = getVertexId(getNext(iter), NULL);
        hash = hashlittle(&vertex, sizeof(int), hash);
      }
      hash &= hashmask(hashbits);
      free(iter);
      
      last = NULL;
      check = ht[hash];
      while (check != NULL && setEqual(check->clique, clique) == FALSE)
      {
        last = check;
        check = check->next;
      }
      if (check == NULL)
      {
        check = (LL*) malloc(sizeof(LL));
        if (check == NULL)
        {
          perror("Error allocating clique structure");
          exit(EXIT_FAILURE);
        }
        check->clique = clique;
        check->next = NULL;
        check->source = CPLUS_SRC;
        if (last != NULL)
          last->next = check;
        else
          ht[hash] = check;
      }
      else
      {
        if (check->source & CPLUS_SRC)
        {
          printf("Duplicate clique in C_PLUS: ");
          printvs(clique);
        }
        check->source |= CPLUS_SRC;
      }
      
      clique = newVertexSet();
      vertex = -1;
      cliquecount++;
    }
    else if (ch == ' ' && vertex >= 0)
    {
      clique = addVertex(getVertex(vertex, NULL), clique);
      vertex = -1;
    }
    else if (ch >= '0' && ch <= '9')
    {
      if (vertex >= 0)
        vertex = vertex * 10 + ch - '0';
      else
        vertex = ch - '0';
    }
    ch = getc(cplus);
  }

  #ifdef DEBUG
  printf("Reading in C_MINUS\n");
  #endif

  ch = getc(cminus);
  vertex = -1;
  while (ch != EOF)
  {
    if (ch == '\n')
    {
      #ifdef DEBUG
      //printf("ccliques[%d] = ", vertex);
      //printvs(ccliques[vertex]->clique);
      #endif
      if (ccliques[vertex]->source & CMINUS_SRC)
        printf("Duplicate clique in C_MINUS: %d\n", vertex);
      ccliques[vertex]->source |= CMINUS_SRC;
      vertex = -1;
    }
    else if (ch >= '0' && ch <= '9')
    {
      if (vertex >= 0)
        vertex = vertex * 10 + ch - '0';
      else
        vertex = ch - '0';
    }
    ch = getc(cminus);
  }
  
  #ifdef DEBUG
  printf("Beginning analysis\n");
  #endif
  
  //Free hash table & analyze results
  for (i = 0; i < hashsize(hashbits); i++)
  {
    check = ht[i];
    while (check != NULL)
    {
      switch(check->source)
      {
        case C_SRC:
          printf("Clique missing from C_MINUS: ");
          printvs(check->clique);
          break;
        case CNEW_SRC:
          printf("Clique missing from C_PLUS: ");
          printvs(check->clique);
          break;
        case C_SRC + CNEW_SRC:
          break;
        case CMINUS_SRC:
          printf("Clique only in C_MINUS (Impossible): ");
          printvs(check->clique);
          break;
        case CMINUS_SRC + C_SRC:
          break;
        case CMINUS_SRC + CNEW_SRC:
          printf("Clique in C_MINUS belongs in C_PLUS (Impossible): ");
          printvs(check->clique);
          break;
        case CMINUS_SRC + CNEW_SRC + C_SRC:
          printf("Extra clique in C_MINUS: ");
          printvs(check->clique);
          break;
        case CPLUS_SRC:
          printf("Extra clique in C_PLUS (not in C_NEW): ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + C_SRC:
          printf("Clique in C_PLUS belongs in C_MINUS: ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + CNEW_SRC:
          break;
        case CPLUS_SRC + CNEW_SRC + C_SRC:
          printf("Extra clique in C_PLUS (also in C): ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + CMINUS_SRC:
          printf("Clique in both C_PLUS and C_MINUS (Impossible): ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + CMINUS_SRC + C_SRC:
          printf("Bad clique in C_PLUS (in C and C_MINUS): ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + CMINUS_SRC + CNEW_SRC:
          printf("Bad clique in C_MINUS (Impossible): ");
          printvs(check->clique);
          break;
        case CPLUS_SRC + CMINUS_SRC + CNEW_SRC + C_SRC:
          printf("Extra clique in both C_PLUS and C_MINUS: ");
          printvs(check->clique);
          break;
        default:
          printf("Something went very wrong...\n");
      }
        
      last = check;
      check = check->next;
      free(last);
    }
  }
  free(ht);
  
  cleanup();
  
  exit(EXIT_SUCCESS);
}
