#ifndef __VERTEX_SET_H
#define __VERTEX_SET_H

#define TRUE -1
#define FALSE 0

#define VERTEX_SET_BLOCK_SIZE 10000

//Define types
typedef struct vs
{
  unsigned int v;
  struct vs* p;
} PVertex;

typedef PVertex* PVertexSet;

typedef struct
{
  PVertexSet next;
  PVertexSet end;
} PVertexSetIterator;

typedef struct
{
  PVertexSet neighborhood_block;
  int num_vertices;
  long num_edges;
  PVertexSet* neighborhood;
  char* marked;
} PGraph;

typedef struct
{
  int u;
  int v;
} PEdge;

PVertexSet current_block;
PVertexSet vs_frame_pointer;

void init();
void cleanup();
void printvs(PVertexSet a);

//PGraph functions
PGraph* newGraph();
void addEdge(int u, int v, PGraph* G);
void freeGraph(PGraph* G);

//PVertex functions
//PVertex getVertex(int u, PGraph* G);
#define getVertex(v, G)             ((PVertex) {v})
//int getVertexId(PVertex u, PGraph* G);
#define getVertexId(u, G)           (u).v
//PVertexSet getNeighborhoodSet(PVertex u, PGraph* G);
#define getNeighborhoodSet(u, G)    (G)->neighborhood[(u).v]
//int getDegree(PVertex u, PGraph* G);
#define getDegree(u, G)             getSize(getNeighborhoodSet(u, G))
//int isMarked(PVertex u, PGraph* G)
#define isMarked(u, G)              (G)->marked[(u).v]
//void mark(PVertex u, PGraph* G)
#define mark(u, G)                  ((G)->marked[(u).v] = TRUE)
//void unmark(PVertex u, PGraph* G)
#define unmark(u, G)                ((G)->marked[(u).v] = FALSE)

//PEdge function
//PEdge newEdge(int u, int v);
#define newEdge(u, v)               ((u) <= (v) ?((PEdge) {(u), (v)}) :((PEdge) {(v), (u)}))

//PVertexSet functions
//void saveFrame()
#define saveFrame()                 (vs_frame_pointer = current_block[1].p)
//void loadFrame()
#define loadFrame()                 (current_block[1].p = vs_frame_pointer)
PVertexSet newVertexSet();
PVertexSet setCopy(PVertexSet a);
PVertexSet deepCopy(PVertexSet a);
PVertexSet addVertex(PVertex u, PVertexSet a);
void removeVertex(PVertex u, PVertexSet a);
PVertexSet setUnion(PVertexSet a, PVertexSet b);
void setIntersection(PVertexSet a, PVertexSet b);
void setMinus(PVertexSet a, PVertexSet b);
//int getSize(PVertexSet a)
#define getSize(a)                  ((a)->p - (a) - 1)
int containsVertex(PVertex u, PVertexSet a);
//int emptySet(VertexSet a);
#define emptySet(a)                 ((a)->p == (a) + 1)
int setEqual(PVertexSet a, PVertexSet b);
int subset(PVertexSet a, PVertexSet b);
//int allAdjacent(PVertex u, PVertexSet b, PGraph* G);
#define allAdjacent(vertex, set, G) subset((set), (G)->neighborhood[(vertex).v])

//PVertexSetIterator functions
PVertexSetIterator* getVertexIterator(PVertexSet a);
//int hasNext(PVertexSetIterator* iter);
#define hasNext(iter)               (iter->next < iter->end)
PVertex getNext(PVertexSetIterator* iter);
//void back_up(PVertexSetIterator* iter);
#define back_up(iter)               (iter->next--)

#endif
