#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "wgc.h"

void addEdge(int u, int v, Graph* G)
{
  VertexLL* i = (VertexLL*) malloc(sizeof(VertexLL));
  i->vertex = v;
  i->next = NULL;

  VertexLL* t = G->nh[u];
  if (t == NULL)
  {
    G->nh[u] = i;
    return;
  }
  
  VertexLL* l = NULL;
  while (t->vertex < v && t->next != NULL)
  {
    l = t;
    t = t->next;
  }
  
  if (t->vertex > v)
  {
    i->next = t;
    if (l != NULL)
      l->next = i;
    else
      G->nh[u] = i;
  }
  else
    t->next = i;
}

Graph* readWeightedGraph(char* filename, double threshold)
{
  Graph* ret;
  FILE* fp;
  int i, edges, verts, u, v, graphedges;
  double w;
  
  fp = fopen(filename, "r");
  fscanf(fp, "%d %d", &verts, &edges);
  ret = (Graph*) malloc(sizeof(Graph));
  ret->nh =  (VertexLL**) calloc(sizeof(VertexLL*), verts);
  ret->num_vertices = verts;

  graphedges = 0;
  for (i = 0; i < edges; i++)
  {
    fscanf(fp, "%d %d %lf", &u, &v, &w);
    if (w >= threshold)
    {
      if (u < v)
        addEdge(u, v, ret);
      else
        addEdge(v, u, ret);
      graphedges++;
    }
  }
  
  ret->num_edges = graphedges;

  fclose(fp);
  return ret;
}

void writeGraph(Graph* G, char* filename)
{
  FILE* fp;
  int i, vertices = G->num_vertices;
  VertexLL* v;
  
  fp = fopen(filename, "w");
  fprintf(fp, "%d %d\n", G->num_vertices, G->num_edges);
  
  for (i = 0; i < vertices; i++)
  {
    v = G->nh[i];
    
    while (v != NULL)
    {
      fprintf(fp, "%d %d\n", i, v->vertex);
      v = v->next;
    }
  }
  
  fclose(fp);
}

void freeGraph(Graph* G)
{
  int v = G->num_vertices;
  int i;
  
  VertexLL* t, * victim;
  
  for (i = 0; i < v; i++)
  {
    t = G->nh[i];
    while (t != NULL)
    {
      victim = t;
      t = t->next;
      free(victim);
    }
  }
  
  free(G->nh);
  free(G);
}

int main(int argc, char** argv)
{
  if (argc != 4)
  {
    fprintf(stderr, "Usage %s [input file] [output file] [threshold]\n", argv[0]);
    exit(1);
  }

  Graph* G = readWeightedGraph(argv[1], strtod(argv[3], NULL));
  writeGraph(G, argv[2]);

  freeGraph(G);
  exit(0);
}

